package com.fractalite.hermes.teldar.parsers;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Currency;
import java.util.HashMap;
import java.util.List;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.xpath.XPathExpressionException;

import org.apache.camel.CamelContext;
import org.apache.camel.Exchange;
import org.apache.camel.InvalidPayloadException;
import org.apache.commons.codec.language.DoubleMetaphone;
import org.codehaus.jackson.map.ObjectMapper;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Component;
import org.xml.sax.SAXException;
import com.fractalite.hermes.teldar.cfg.ContentProvider;
import com.fractalite.models.commerce.PriceScope;
import com.fractalite.models.commerce.QuoteTemplate;
import com.fractalite.models.common.Address;
import com.fractalite.models.common.CodeNamePair;
import com.fractalite.models.common.Company;
import com.fractalite.models.common.Location;
import com.fractalite.models.common.Money;
import com.fractalite.models.travel.stay.Board;
import com.fractalite.models.travel.stay.Hotel;
import com.fractalite.models.travel.stay.HotelStay;
import com.fractalite.models.travel.stay.Image;
import com.fractalite.models.travel.stay.RoomProduct;
import com.fractalite.models.travel.stay.RoomType;
import com.fractalite.models.travel.stay.StaySearchResults;
import com.gekko_holding.webservice.v2_4.AvailabilityResponse;
import com.gekko_holding.webservice.v2_4.CancellationFeesPolicy;
import com.gekko_holding.webservice.v2_4.Facility;
import com.gekko_holding.webservice.v2_4.HotelAvailabilityResponse;
import com.gekko_holding.webservice.v2_4.HotelResponse;
import com.gekko_holding.webservice.v2_4.Offer;
import com.gekko_holding.webservice.v2_4.RoomOffer;

import jdk.internal.org.jline.utils.Log;

@Component
public class ParseTeldarHotelsPriceResponse extends TeldarParsers {
	protected static Logger logger = LoggerFactory.getLogger(ParseTeldarHotelsPriceResponse.class);
	
	static HashMap<String, String> boardMapping= new HashMap<String, String>();
	static {
		boardMapping.put("BRKF", "Petit-déjeuner inclus");
		boardMapping.put("RONL", "Chambre seule");
		boardMapping.put("DIN", "Diner Gratuit");
		boardMapping.put("FULL", "Pension complète");

		boardMapping.put("HALF", "Demi-Pension");

		boardMapping.put("ALLI", "Tout-inclus");

		boardMapping.put("NONE", "Aucun");


	}

	private CamelContext camelContext;

	@Override
	public void process(Exchange exchange) throws InvalidPayloadException, ParserConfigurationException, SAXException,
			IOException, XPathExpressionException {

		//logger.info("\n Invoking TELDAR ParseHotelsPriceResponse Processor....... " + exchange.getIn().getBody());
		HotelAvailabilityResponse resp = exchange.getIn().getBody(HotelAvailabilityResponse.class);
		/* Converting to a list of hotels. */
		camelContext = exchange.getContext();
		List<Hotel> result = printHotelInfo(resp.getAvailResponse(),
				exchange.getProperty("hermesquery", QuoteTemplate.class));
		/* Preparing response. */

		StaySearchResults staySearchResults = new StaySearchResults();

		staySearchResults.setHotels(result);
		ObjectMapper mapper = new ObjectMapper();
		staySearchResults.setQuery(exchange.getProperty("hermesquery", QuoteTemplate.class));
		exchange.getIn().setBody(staySearchResults, StaySearchResults.class);
	}

	private static String objectToJson(StaySearchResults staySearchResults) {
		ObjectMapper objectMapper = new ObjectMapper();
		String res = "";
		try {
			res = objectMapper.writeValueAsString(staySearchResults);
		} catch (Exception e) {
			e.printStackTrace();
		}
		return res;
	}

	private List<Hotel> printHotelInfo(AvailabilityResponse teldarHotel, QuoteTemplate quoteTemplate)
			throws XPathExpressionException {

		HashMap<String, String> cntry = new HashMap<String, String>();
		
		
		HashMap<String, String> giataMapping = new HashMap<String, String>();
		if(quoteTemplate != null && quoteTemplate.getTrip()!=null && quoteTemplate.getTrip().getContent(HotelStay.class) !=null && quoteTemplate.getTrip().getContent(HotelStay.class).get(0) !=null && quoteTemplate.getTrip().getContent(HotelStay.class).get(0).getGiatacode() !=null &&  quoteTemplate.getTrip().getContent(HotelStay.class).get(0).getGiatacode().getGekko_teldar()!=null)
		{
			List<String> hotelcodes = quoteTemplate.getTrip().getContent(HotelStay.class).get(0).getGiatacode().getGekko_teldar();
			for (String code : hotelcodes) {
				String[] codes = code.split("-");
				String supplierCode = codes[0];
				String giataCode = codes[1];
				giataMapping.put(supplierCode, giataCode);
			}
		}
				
		
		//logger.info("giataccodTeldar" + giataMapping.size());
		
		logger.info("Teldar ask for refundable? " +quoteTemplate.getRefundable());


		List<String> lstCode = new ArrayList<String>(Arrays.asList("AD", "AE", "AF", "AG", "AI", "AL", "AM", "AO", "AR",
				"AS", "AT", "AU", "AW", "AZ", "BA", "BB", "BD", "BE", "BF", "BG", "BH", "BI", "BJ", "BL", "BM", "BN",
				"BO", "BQ", "BR", "BS", "BT", "BW", "BY", "BZ", "CA", "CD", "CF", "CG", "CH", "CI", "CK", "CL", "CM",
				"CN", "CO", "CR", "CU", "CV", "CW", "CY", "CZ", "DE", "DJ", "DK", "DM", "DO", "DZ", "EC", "EE", "EG",
				"EH", "ER", "ES", "ET", "FI", "FJ", "FK", "FM", "FO", "FR", "GA", "GB", "GD", "GE", "GF", "GH", "GI",
				"GL", "GM", "GN", "GP", "GQ", "GR", "GT", "GU", "GW", "GY", "HK", "HN", "HR", "HT", "HU", "ID", "IE",
				"IL", "IN", "IQ", "IR", "IS", "IT", "JM", "JO", "JP", "KE", "KG", "KH", "KI", "KM", "KN", "KR", "KW",
				"KY", "KZ", "LA", "LB", "LC", "LI", "LK", "LR", "LS", "LT", "LU", "LV", "LY", "MA", "MC", "MD", "ME",
				"MF", "MG", "MK", "ML", "MM", "MN", "MO", "MP", "MQ", "MR", "MS", "MT", "MU", "MV", "MW", "MX", "MY",
				"MZ", "NA", "NC", "NE", "NF", "NG", "NI", "NL", "NO", "NP", "NR", "NU", "NZ", "OM", "PA", "PE", "PF",
				"PG", "PH", "PK", "PL", "PM", "PR", "PS", "PT", "PW", "PY", "QA", "RE", "RO", "RS", "RU", "RW", "SA",
				"SB", "SC", "SD", "SE", "SG", "SI", "SK", "SL", "SM", "SN", "SO", "SR", "SS", "ST", "SV", "SX", "SY",
				"SZ", "TC", "TD", "TG", "TH", "TJ", "TL", "TM", "TN", "TO", "TR", "TT", "TW", "TZ", "UA", "UG", "US",
				"UY", "UZ", "VC", "VE", "VG", "VI", "VN", "VU", "WF", "WS", "XO", "YE", "ZA", "ZM", "ZW"));
		List<String> lstName = new ArrayList<String>(Arrays.asList("Andorra", "United Arab Emirates", "Afghanistan",
				"Antigua", "Anguilla", "Albania", "Armenia", "Angola", "Argentina", "AmericanS amoa", "Austria",
				"Australia", "Aruba", "Aserbaidshan", "Bosnia", "Barbados", "Bangladesh", "Belgium", "BurkinaFaso",
				"Bulgaria", "Bahrain", "Burundi", "Benin", "Saint-Barthélemy", "Bermuda", "Brunei Darussalam",
				"Bolivia", "Bonaire,Sint Eustatius", "Brazil", "Bahamas", "Bhutan", "Botswana", "Belarus", "Belize",
				"Canada", "Congo,Democratic Republic", "Central African Republic", "Congo", "Switzerland", "IvoryCoast",
				"CookIslands", "Chile", "Cameroon", "China", "Colombia", "CostaRica", "Cuba", "CapeVerde", "Curaçao",
				"Cyprus", "CzechRepublic", "Germany", "Djibouti", "Denmark", "Dominica", "Dominican Republic",
				"Algeria", "Ecuador", "Estland", "Egypt", "WesternSahara", "Eritrea", "Spain", "Ethiopia", "Finland",
				"Fiji", "Islands", "FederatedS tatesofMicronesia", "FaeroeIslands", "France", "Gabon", "UnitedKingdom",
				"Grenada", "Georgia", "FrenchGuiana", "Ghana", "Gibraltar", "Greenland", "Gambia", "Guinea",
				"Guadeloupe", "EquatorialGuinea", "Greece", "Guatemala", "Guam", "Guinea-Bissau", "Guyana", "HongKong",
				"Honduras", "Croatia", "Haiti", "Hungary", "Indonesia", "Eire/Ireland", "Israel", "India", "Iraq",
				"Iran", "Iceland", "Italy", "Jamaica", "Jordan", "Japan", "Kenya", "Kyrgystan", "Cambodia", "Kiribati",
				"Comoros", "St.KittsandNevis", "Korea,Republicof", "Kuwait", "CaymanIslands", "Kazakstan",
				"LaoPeople'sDemocraticRepublic", "Lebanon", "SaintLucia", "Liechtenstein", "SriLanka", "Liberia",
				"Lesotho", "Lithuania", "Luxembourg", "Latvia", "Libya", "Morocco", "Monaco", "Moldova", "Montenegro",
				"Saint-Martin", "Madagascar", "Macedonia(FYROM)", "Mali", "Myanmar", "Mongolia", "Macau",
				"NorthernMarianaIslands", "Martinique", "Mauritania", "Montserrat", "Malta", "Mauritius", "Maldives",
				"Malawi", "Mexico", "Malaysia", "Mozambique", "Namibia", "NewCaledonia", "Niger", "NorfolkIsland",
				"Nigeria", "Nicaragua", "Netherlands", "Norway", "Nepal", "Nauru", "Niue", "NewZealand", "Oman",
				"Panama", "Peru", "FrenchPolynesia", "PapuaNewGuinea", "Philippines", "Pakistan", "Poland",
				"St.PierreandMiquelon", "PuertoRico", "Palestine", "Portugal", "Palau", "Paraguay", "Qatar", "Reunion",
				"Romania", "Serbia", "RussianFederation", "Rwanda", "SaudiArabia", "SolomonIslands", "Seychelles",
				"Sudan", "Sweden", "Singapore", "Slovenia", "Slovakia", "SierraLeone", "SanMarino", "Senegal",
				"Somalia", "Suriname", "SouthSudan", "SaoTomeandPrincipe", "ElSalvador", "SintMaarten",
				"SyrianArabRepublic", "Swaziland", "TurksandCaicosIslands", "Chad", "Togo", "Thailand", "Tajikistan",
				"EastTimor", "Turkmenistan", "Tunisia", "Tonga", "Turkey", "TrinidadandTobago",
				"Taiwan,ProvinceofChina", "Tanzania", "Ukraine", "Uganda", "USA", "Uruguay", "Uzbekistan",
				"SaintVincentandtheGrenadines", "Venezuela", "VirginIslands,British", "VirginIslands,U.S.", "VietNam",
				"Vanuatu", "WallisandFutunaIslands", "Samoa", "OtherCountries", "Yemen", "SouthAfrica", "Zambia",
				"Zimbabwe"));
		
		
		

		for (int i = 0; i < 228; i++) {
			cntry.put(lstCode.get(i), lstName.get(i));

		}
		List<Hotel> hermesHotels = new ArrayList<Hotel>();
		if (teldarHotel != null && teldarHotel.getHotelResponse() != null
				&& !teldarHotel.getHotelResponse().isEmpty()) {
			
			logger.info("teldarhotelsizeee" + teldarHotel + "̣ size " + teldarHotel.getHotelResponse().size());
			for (HotelResponse respHotel : teldarHotel.getHotelResponse()) {
				List<Image> images = new ArrayList<Image>();

				Hotel hermesHotel = new Hotel();
				if (respHotel.getCode() != null) {
					hermesHotel.setCode(respHotel.getCode());
					if(giataMapping !=null && giataMapping.keySet()!=null && giataMapping.keySet().size()>0)
						hermesHotel.setGiataCode(giataMapping.get(respHotel.getCode()));

				}
				// Hotel Chain

				if (respHotel.getChainCode() != null) {
					hermesHotel.setChain(new Company(respHotel.getChainCode()));
				}

				if (respHotel.getName() != null) {
					hermesHotel.setName(respHotel.getName());

				}
				
				if (respHotel.getThumbnailUrl() != null) {
					//hermesHotel.setName(respHotel.getName());
					
					Image im = new Image("1", "Search", respHotel.getThumbnailUrl(), respHotel.getThumbnailUrl(), ((short) 127), ((short) 127));
					images.add(im);
					hermesHotel.setImages(images );
				}
				
				//respHote
				

				hermesHotel.setRecommended(true);

				String cd = respHotel.getCity().getCode().substring(0, 2);
				String nm = cntry.get(cd);

				com.fractalite.models.common.Address hermesAddress = new Address(respHotel.getAddress(),
						respHotel.getZipcode(),
						new Location(respHotel.getCity().getCode().substring(2, 5),
								respHotel.getCity().getCode().substring(2, 5)),
						new Location(respHotel.getCity().getCode().substring(0, 5), nm));
				hermesHotel.setAddress(hermesAddress);

				// lattitude
				if (respHotel.getLatitude() != null) {
					hermesHotel.setLatitude(respHotel.getLatitude().floatValue());
				}
				// longitude
				if (respHotel.getLongitude() != null) {
					hermesHotel.setLongitude(respHotel.getLongitude().floatValue());
				}

				// Hotel Rating
				if (respHotel.getRating() != null) {
					HotelRatingg rating;
					if (respHotel.getRating().getCode().equals("UNKNOWN")
							|| respHotel.getRating().getCode().equals("UNRATED")) {
						rating = HotelRatingg.valueOf(respHotel.getRating().getCode());
						hermesHotel.setRating(new CodeNamePair(""+0, "0ST"));

					} else {
						rating = HotelRatingg.valueOf("_" + respHotel.getRating().getCode());
						hermesHotel.setRating(new CodeNamePair(String.valueOf(rating.value()), rating.value() + "ST"));

					}
					
				}

				// distance to ---> meta
				if (respHotel.getDistance() > 0) {
					hermesHotel.getMetas().put("distance", Arrays.asList(String.valueOf(respHotel.getDistance())));
				}
				// price---> room
			
				if (respHotel.getFacilities() != null && respHotel.getFacilities().getFacilities() != null
						&& !respHotel.getFacilities().getFacilities().isEmpty()) {
					/*
					 * list of all facilities content[e.g: Accès Internet] not code[e.g : SVGEKKO-1]
					 */
					List<String> facilities = new ArrayList<String>();
					for (Facility respFacility : respHotel.getFacilities().getFacilities()) {
						if (respFacility.getCode() != null && !respFacility.getCode().isEmpty()) {
							//com.fractalite.hermes.teldar.parsers.Facility hermesFacility = com.fractalite.hermes.teldar.parsers.Facility
							//		.valueOf(respFacility.getCode().replace("-", "_"));
						//	if (hermesFacility != null) {
								facilities.add(respFacility.getValue());
						//	}
						}
					}
					hermesHotel.setAmenities(facilities);
				}

				/*
				if (respHotel.getOffers().getOffer().get(0).getCancellationFeesPolicy() != null
						&& !respHotel.getOffers().getOffer().get(0).getCancellationFeesPolicy().isEmpty()) {
					HashMap<String, HashMap<String, String>> cancellationFees = new HashMap<String, HashMap<String, String>>();

					for (CancellationFeesPolicy respHotelBook : respHotel.getOffers().getOffer().get(0)
							.getCancellationFeesPolicy()) {

						
						if (respHotelBook.getFromDate() != null) {
							cancellationFees.put(respHotelBook.getFromDate(), new HashMap<String, String>());
						}
						if (respHotelBook.getToDate() != null) {
							cancellationFees.get(respHotelBook.getFromDate()).put("ToDate", respHotelBook.getToDate());
						}

						if (respHotelBook.getCurrency() != null) {
							cancellationFees.get(respHotelBook.getCurrency());
						}

						if (respHotelBook.getPrice() != null) {
							cancellationFees.get(respHotelBook.getFromDate()).put("Price", respHotelBook.getPrice());
						}

						else {
							cancellationFees.put("allDates", new HashMap<String, String>());

							if (respHotelBook.getCurrency() != null) {
								cancellationFees.get("allDates").put("Currency", respHotelBook.getCurrency());
							}

							if (respHotelBook.getPrice() != null) {
								cancellationFees.get("allDates").put("Price", respHotelBook.getPrice());
							}
						}
					}
					
				}*/
				//hermesHotel.getHotelPolicy()
				// offers
				List<RoomType> hermesRoomTypes = new ArrayList<RoomType>();
				List<RoomProduct> hermesRoomProducts = new ArrayList<RoomProduct>();
				String allBoard="";
				
				if (respHotel.getOffers() != null && respHotel.getOffers().getOffer() != null
						&& !respHotel.getOffers().getOffer().isEmpty()) {
			for (Offer offer : respHotel.getOffers().getOffer()) {
				for (RoomOffer roomOffer : offer.getRoomOffers().getRoomOffer()) {
					if (roomOffer != null) {
						if (roomOffer.getBoardType() != null
								&& roomOffer.getBoardType().getCode() != null
								&& roomOffer.getBoardType().getValue() != null) {
							String board;
								if(boardMapping.keySet().contains(roomOffer.getBoardType().getCode()))
									board=boardMapping.get(roomOffer.getBoardType().getCode());
								else
									board= roomOffer.getBoardType().getCode()+":"+roomOffer.getBoardType().getValue();
								logger.info("Room "+roomOffer.getRoomType().getValue()+" has board "+board);

								if(allBoard.contains(board))
										continue;
								else
									allBoard=board+","+allBoard;
											
						}
					}
				}
					}
				}
				if(allBoard.length()> 0)
					allBoard = allBoard.substring(0, allBoard.length() - 1);
			
				
				logger.info("------------- All board "+allBoard);
				
				if (respHotel.getOffers() != null && respHotel.getOffers().getOffer() != null
						&& !respHotel.getOffers().getOffer().isEmpty()) {
					for (Offer offer : respHotel.getOffers().getOffer()) {
						if(quoteTemplate.getRefundable() && offer.getOfferCode().startsWith("NRF"))
							continue;
						
						
						
						
						
						RoomProduct roomproduct = new RoomProduct();
						for (RoomOffer roomOffer : offer.getRoomOffers().getRoomOffer()) {
							if (roomOffer != null) {
								// RoomProduct hermesRoomProduct;
								RoomType hermesRoomType;
								if (roomOffer.getRoomType() != null && roomOffer.getRoomType().getCode() != null
										&& roomOffer.getRoomType().getValue() != null
										&& roomOffer.getAdultsCount() > 0) {
									hermesRoomType = new RoomType(roomOffer.getRoomType().getCode(),
											roomOffer.getAdultsCount(), roomOffer.getChildrenCount(),
											roomOffer.getRoomType().getValue());
									hermesRoomTypes.add(hermesRoomType);
									if (offer.getOfferCode() != null) {
										if (roomOffer.getBoardType() != null
												&& roomOffer.getBoardType().getCode() != null
												&& roomOffer.getBoardType().getValue() != null) {
											roomproduct = new RoomProduct(offer.getOfferCode(), hermesRoomType,
													roomOffer.getAdultsCount() + roomOffer.getChildrenCount(),
													new CodeNamePair(roomOffer.getBoardType().getCode(),
															allBoard));
										} else {
											roomproduct = new RoomProduct(offer.getOfferCode(), hermesRoomType,
													roomOffer.getAdultsCount() + roomOffer.getChildrenCount());
										}
									}
								}
							}
						}

						if (offer.getCurrency() != null && offer.getOfferCode() != null
								&& offer.getTotalPrice().getValue() != null) {
							if (offer.getRoomOffers() != null && offer.getRoomOffers() != null
									&& offer.getRoomOffers().getRoomOffer() != null
									&& !offer.getRoomOffers().getRoomOffer().isEmpty()) {
								for (RoomOffer respRoomOffer : offer.getRoomOffers().getRoomOffer()) {
									if (respRoomOffer.getRoomPrice() != null
//                                             && respRoomOffer.getRoomPrice().getCommission() != null
											&& respRoomOffer.getRoomPrice().getCurrency() != null
											&& respRoomOffer.getRoomPrice().getValue() != null) {

										Double finalPrice = respRoomOffer.getRoomPrice().getValue();
										logger.info("------- for hotel "+respHotel.getName()+" revceived price from teldar "+respRoomOffer.getRoomPrice().getValue());
										Money totalPrice = new Money(finalPrice,
												Currency.getInstance(respRoomOffer.getRoomPrice().getCurrency()));
										roomproduct.setPrice(PriceScope.PURCHASE_TOTAL_PRICE, totalPrice);

										hermesRoomProducts.add(roomproduct);
										break;
									}
								}
							}
						}
						hermesHotel.setRoomTypes(hermesRoomTypes);
						hermesHotel.addProducts(hermesRoomProducts);
						break;
					}
				}
				hermesHotel.addMeta("source", ContentProvider.PROVIDER_ID);
				hermesHotels.add(hermesHotel);
			}
		}
	//	logger.info("hermessssize" + hermesHotels.size());

		return hermesHotels;
	}
	

	    public static void main(String[] args) {
	        DoubleMetaphone metaphone = new DoubleMetaphone();

	        String str1 = "board full";
	        String str2 = "board half";

	        String metaphone1 = metaphone.doubleMetaphone(str1);
	        String metaphone2 = metaphone.doubleMetaphone(str2);

	        System.out.println("Metaphone of str1: " + metaphone1);
	        System.out.println("Metaphone of str2: " + metaphone2);

	        if (metaphone.isDoubleMetaphoneEqual(str1, str2)) {
	            System.out.println("The strings sound similar!");
	        } else {
	            System.out.println("The strings do not sound similar.");
	        }
	    }

}
