package com.fractalite.hermes.teldar.services;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import javax.xml.datatype.DatatypeConfigurationException;

import org.apache.camel.Exchange;
import org.apache.camel.InvalidPayloadException;
import org.eclipse.jetty.util.log.Log;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.fractalite.hermes.teldar.Marshaller.MarshallingJAXB;
import com.fractalite.hermes.teldar.cfg.TeldarRouteBuilder;
import com.fractalite.models.commerce.QuoteTemplate;
import com.fractalite.models.travel.PaxType;
import com.fractalite.models.travel.Traveller;
import com.fractalite.models.travel.stay.HotelStay;
import com.fractalite.models.travel.stay.Occupancy2;
import com.fractalite.models.travel.stay.Room;
import com.gekko_holding.webservice.v2_4.AvailabilityCriteria;
import com.gekko_holding.webservice.v2_4.AvailabilityCriteria.RoomCriterias;
import com.gekko_holding.webservice.v2_4.Child;
import com.gekko_holding.webservice.v2_4.CityDestination;
import com.gekko_holding.webservice.v2_4.CustomerIdentification;
import com.gekko_holding.webservice.v2_4.DestinationCriteria;
import com.gekko_holding.webservice.v2_4.GeoCodeDestination;
import com.gekko_holding.webservice.v2_4.HotelAvailability;
import com.gekko_holding.webservice.v2_4.HotelCodeDestination;
import com.gekko_holding.webservice.v2_4.HotelCodeListDestination;
import com.gekko_holding.webservice.v2_4.RoomPlan;
import com.gekko_holding.webservice.v2_4.RoomPlan.Children;
import com.google.common.collect.Lists;


public class SearchTeldarHotelPriceRequest extends TeldarServices {

	protected static Logger logger = LoggerFactory.getLogger(SearchTeldarHotelPriceRequest.class);

	private static HashMap<String, String> destMapping = new HashMap<String, String>();
	
	static {
	//	FR90922
		destMapping.put("8714", "FR90922");
	}
	
	
	@Override
	public void process(Exchange exchange)
			throws InvalidPayloadException, ParseException, DatatypeConfigurationException {

		 logger.trace("\n Invoking Teldarrrrrrrr SearchHotelsPriceRequest Processor.......");
		QuoteTemplate q = exchange.getIn().getMandatoryBody(QuoteTemplate.class);
		HotelStay hotelstay = q.getTrip().getContent(HotelStay.class).get(0);
		exchange.setProperty("hermesquery", q);
		//HotelAvailability teldarReq = getList(hotelstay);
		//logger.info("\n End teldargooooo SearchHotelsPriceRequest Processor......." + MarshallingJAXB.printXML(teldarReq));
		//exchange.getIn().setBody(teldarReq, HotelAvailability.class);
		List<List<String>> subteltardcodes= new ArrayList<List<String>>();
		List<HotelAvailability> listHotelAvailability =new ArrayList<HotelAvailability>();

		if(hotelstay.getGiatacode() != null && hotelstay.getGiatacode().getGekko_teldar() !=null && hotelstay.getGiatacode().getGekko_teldar().size()> 0)
		{
			List<String> teltardcode = hotelstay.getGiatacode().getGekko_teldar();
			subteltardcodes= Lists.partition(teltardcode,290);
			
			logger.info("the first list size "+teltardcode.size());
			logger.info("the number of partitions "+subteltardcodes.size());
			int i=0;
			for(List<String> subteltardcode:subteltardcodes) {
				logger.info("the "+(++i)+"  partition "+subteltardcode.size());

				HotelAvailability teldarReq = getList(hotelstay,subteltardcode);
				listHotelAvailability.add(teldarReq);
			}
		}
		else // GPS  or mapping 
		{
	
			
			HotelAvailability teldarReq = getList(hotelstay,null);
			listHotelAvailability.add(teldarReq);

		}
		

		
		
		exchange.getIn().setBody(listHotelAvailability);
	}

	public HotelAvailability getList(HotelStay hotelstay,List<String> teltardcode ) throws ParseException, DatatypeConfigurationException {
		HotelAvailability hotelavailability = new HotelAvailability();
		CustomerIdentification customeridentification = new CustomerIdentification();

		customeridentification.setClientId(TeldarRouteBuilder.clientId);
		customeridentification.setPassword(TeldarRouteBuilder.password);
		customeridentification.setCustomerKey(TeldarRouteBuilder.customerKey);


		AvailabilityCriteria avaCriteria = new AvailabilityCriteria();
		SimpleDateFormat formatter = new SimpleDateFormat("yyyy-mm-dd");

		hotelavailability.setIdentification(customeridentification);

		DestinationCriteria destinationcriteria = new DestinationCriteria();
	
		String desCode = hotelstay.getHotel().getAddress().getLocality().getCode();
		logger.info("loccccc" + desCode);
		if(destMapping.keySet().contains(desCode))
		{
			CityDestination city = new CityDestination();
			city.setCode(destMapping.get(desCode));
			Log.info("---- Searc by dest code "+destMapping.get(desCode));
			//city.setStandard(desCode);
			destinationcriteria.setCity(city );
		}
		else if (desCode.startsWith("geo")) {
			String[] geos = desCode.substring(4).split(";");
			logger.info("geo code is "+desCode.substring(4));
			GeoCodeDestination geo = new GeoCodeDestination();
			geo.setLongitude(Double.parseDouble(geos[1]));
			geo.setLatitude(Double.parseDouble(geos[0]));
			geo.setRange(Double.parseDouble("1"));
			destinationcriteria.setGeoCode(geo);
			avaCriteria.setDestinationCriteria(destinationcriteria);
			logger.info("  : "+geo.getLongitude());
			logger.info("latitude : "+geo.getLatitude());

		}
		else if(!desCode.matches("-?\\d+")) //search by destnitation 
		{
			logger.info("Search by destination code "+desCode);

			CityDestination city = new CityDestination();
			city.setCode(desCode);
			city.setRange(30d);
			city.setStandard("locode");
			destinationcriteria.setCity(city );;
			avaCriteria.setDestinationCriteria(destinationcriteria);
		}
		
		else {
			// Search By hotels codes

			HotelCodeListDestination hotelsList = new HotelCodeListDestination();
		//	int i=0;
			for (String hotelcodes : teltardcode) {
				String[] codes = hotelcodes.split("-");
				String supplierCode = codes[0];
				String giataCode = codes[1];
				HotelCodeDestination hcode = new HotelCodeDestination();
				hcode.setCode(supplierCode);
				hotelsList.getHotel().add(hcode);
			//	logger.info("i m in hotel code   "+hcode+" nbr "+(++i));
				//if(i>298)
				//	break;
				//i++;
				
			}

			destinationcriteria.setHotels(hotelsList);
			avaCriteria.setDestinationCriteria(destinationcriteria);
		}

		hotelavailability.setLanguage(language);
		avaCriteria.setCheckIn(hotelstay.getCheckin().toString());
		avaCriteria.setCheckOut(hotelstay.getCheckout().toString());
	
		RoomCriterias crit = new RoomCriterias();
		String roo = "";
		for (Room chambre : hotelstay.getRooms()) {

			Occupancy2 occupancy = new Occupancy2();
			RoomPlan roomoffers = new RoomPlan();

			int nbreOfAdult = 0;
		
			Children chlidren = new Children();
			
			for (Traveller traveller : chambre.getGuests()) {

				if (traveller.getClassification() == PaxType.ADULT)
					nbreOfAdult++;
				else if (traveller.getClassification() == PaxType.CHILD) {
		
					Child child = new Child();
					child.setAge(traveller.getAge().intValue());
					chlidren.getChild().add(child);
				}

			}
			occupancy.setAdults(nbreOfAdult);
			roomoffers.setRoomType(roo);
			roomoffers.setAdultsCount(nbreOfAdult);
			roomoffers.setChildren(chlidren);
			roomoffers.getRoomType();
			crit.getRoomPlan().add(roomoffers);

		}

		avaCriteria.setRoomCriterias(crit);

		hotelavailability.setAvailCriteria(avaCriteria);


		return hotelavailability;

	}

}
